=== modified file 'lib/lp/services/job/interfaces/job.py'
--- lib/lp/services/job/interfaces/job.py	2009-07-22 14:54:40 +0000
+++ lib/lp/services/job/interfaces/job.py	2009-07-31 17:51:13 +0000
@@ -14,7 +14,7 @@
     ]
 
 
-from zope.interface import Interface
+from zope.interface import Interface, Attribute
 from zope.schema import Choice, Datetime, Int, Text
 from lazr.enum import DBEnumeratedType, DBItem
 
@@ -107,5 +107,14 @@
         :param oops: The oops produced by this Job.
         """
 
+    user_error_types = Attribute(
+        'A tuple of exception classes which result from user error.')
+
+    def notifyUserError(e):
+        """Notify interested parties that this job encountered a user error.
+
+        :param e: The exception encountered by this job.
+        """
+
     def run():
         """Run this job."""

=== modified file 'lib/lp/services/job/runner.py'
--- lib/lp/services/job/runner.py	2009-07-28 19:34:16 +0000
+++ lib/lp/services/job/runner.py	2009-07-31 19:32:36 +0000
@@ -32,8 +32,14 @@
     """
     delegates(IJob, 'job')
 
+    user_error_types = ()
+
     def getOopsRecipients(self):
         """Return a list of email-ids to notify about oopses."""
+        return self.getErrorRecipients()
+
+    def getErrorRecipients(self):
+        """Return a list of email-ids to notify about user errors."""
         return []
 
     def getOopsMailController(self, oops_id):
@@ -44,12 +50,28 @@
         recipients = self.getOopsRecipients()
         if len(recipients) == 0:
             return None
+        subject = 'Launchpad internal error'
         body = (
             'Launchpad encountered an internal error during the following'
             ' operation: %s.  It was logged with id %s.  Sorry for the'
             ' inconvenience.' % (self.getOperationDescription(), oops_id))
         from_addr = config.canonical.noreply_from_address
-        return MailController(from_addr, recipients, 'NullJob failed.', body)
+        return MailController(from_addr, recipients, subject, body)
+
+    def getUserErrorMailController(self, e):
+        """Return a MailController for notifying about user errors.
+
+        Return None if there is no-one to notify.
+        """
+        recipients = self.getErrorRecipients()
+        if len(recipients) == 0:
+            return None
+        subject = 'Launchpad error while %s' % self.getOperationDescription()
+        body = (
+            'Launchpad encountered an error during the following'
+            ' operation: %s.  %s' % (self.getOperationDescription(), str(e)))
+        from_addr = config.canonical.noreply_from_address
+        return MailController(from_addr, recipients, subject, body)
 
     def notifyOops(self, oops):
         """Report this oops."""
@@ -58,6 +80,13 @@
             return
         ctrl.send()
 
+    def notifyUserError(self, e):
+        """See `IRunnableJob`."""
+        ctrl = self.getUserErrorMailController(e)
+        if ctrl is None:
+            return
+        ctrl.send()
+
 
 class JobRunner(object):
     """Runner of Jobs."""
@@ -103,6 +132,8 @@
                 self.runJob(job)
             except LeaseHeld:
                 self.incomplete_jobs.append(job)
+            except job.user_error_types, e:
+                job.notifyUserError(e)
             except Exception:
                 info = sys.exc_info()
                 errorlog.globalErrorUtility.raising(info)

=== modified file 'lib/lp/services/job/tests/test_runner.py'
--- lib/lp/services/job/tests/test_runner.py	2009-07-28 19:34:16 +0000
+++ lib/lp/services/job/tests/test_runner.py	2009-07-31 18:33:37 +0000
@@ -25,12 +25,16 @@
 
     JOB_COMPLETIONS = []
 
-    def __init__(self, completion_message, oops_recipients=None):
+    def __init__(self, completion_message, oops_recipients=None,
+                 error_recipients=None):
         self.message = completion_message
         self.job = Job()
         self.oops_recipients = oops_recipients
         if self.oops_recipients is None:
             self.oops_recipients = []
+        self.error_recipients = error_recipients
+        if self.error_recipients is None:
+            self.error_recipients = []
 
     def run(self):
         NullJob.JOB_COMPLETIONS.append(self.message)
@@ -38,6 +42,9 @@
     def getOopsRecipients(self):
         return self.oops_recipients
 
+    def getErrorRecipients(self):
+        return self.error_recipients
+
     def getOperationDescription(self):
         return 'appending a string to a list'
 
@@ -145,6 +152,38 @@
             notification.get_payload(decode=True))
         self.assertNotIn('Fake exception.  Foobar, I say!',
                          notification.get_payload(decode=True))
+        self.assertEqual('Launchpad internal error', notification['subject'])
+
+    def test_runAll_mails_user_errors(self):
+        """User errors should be mailed out without oopsing.
+
+        User errors are identified by the RunnableJob.user_error_types
+        attribute.  They do not cause an oops to be recorded, and their
+        error messages are mailed to interested parties verbatim.
+        """
+        job_1, job_2 = self.makeTwoJobs()
+        class ExampleError(Exception):
+            pass
+        def raiseError():
+            raise ExampleError('Fake exception.  Foobar, I say!')
+        job_1.run = raiseError
+        job_1.user_error_types = (ExampleError,)
+        job_1.error_recipients = ['jrandom@example.org']
+        runner = JobRunner([job_1, job_2])
+        reporter = errorlog.globalErrorUtility
+        old_oops = reporter.getLastOopsReport()
+        runner.runAll()
+        self.assertNoNewOops(old_oops)
+        notifications = pop_notifications()
+        self.assertEqual(1, len(notifications))
+        body = notifications[0].get_payload(decode=True)
+        self.assertEqual(
+            'Launchpad encountered an error during the following operation:'
+            ' appending a string to a list.  Fake exception.  Foobar, I say!',
+            body)
+        self.assertEqual(
+            'Launchpad error while appending a string to a list',
+            notifications[0]['subject'])
 
     def test_runAll_requires_IRunnable(self):
         """Supplied classes must implement IRunnableJob.

=== modified file 'lib/lp/testing/__init__.py'
--- lib/lp/testing/__init__.py	2009-07-22 02:26:16 +0000
+++ lib/lp/testing/__init__.py	2009-07-31 18:33:37 +0000
@@ -27,6 +27,7 @@
 from zope.security.proxy import (
     isinstance as zope_isinstance, removeSecurityProxy)
 
+from canonical.launchpad.webapp import errorlog
 from lp.codehosting.bzrutils import ensure_base
 from lp.codehosting.vfs import branch_id_to_path, get_multi_server
 from canonical.config import config
@@ -176,6 +177,14 @@
                                  ', '.join([repr(event) for event in events]))
         return result
 
+    def assertNoNewOops(self, old_oops):
+        """Assert that no oops has been recorded since old_oops."""
+        oops = errorlog.globalErrorUtility.getLastOopsReport()
+        if old_oops is None:
+            self.assertIs(None, oops)
+        else:
+            self.assertEqual(oops.id, old_oops.id)
+
     def assertSqlAttributeEqualsDate(self, sql_object, attribute_name, date):
         """Fail unless the value of the attribute is equal to the date.
 

